Package: codelink
Version: 1.78.0
Title: Manipulation of Codelink microarray data
Author: Diego Diez
Maintainer: Diego Diez <diego10ruiz@gmail.com>
Depends: R (>= 2.10), BiocGenerics (>= 0.3.2), methods, Biobase (>=
        2.17.8), limma
Imports: annotate
Suggests: genefilter, parallel, knitr
LazyLoad: yes
Description: This package facilitates reading, preprocessing and manipulating
 Codelink microarray data. The raw data must be exported as text file using the
 Codelink software.
License: GPL-2
Collate: Codelink-class.R CodelinkSet-class.R file.R data.R norm.R
        plot.R CodelinkSet-methods.R CodelinkSet-tools.R
        CodelinkSetUnique-class.R CodelinkSetUnique-methods.R filter.R
        cluster.R
biocViews: Microarray, OneChannel, DataImport, Preprocessing
ByteCompile: yes
VignetteBuilder: knitr
URL: https://github.com/ddiez/codelink
BugReports: https://github.com/ddiez/codelink/issues
git_url: https://git.bioconductor.org/packages/codelink
git_branch: RELEASE_3_22
git_last_commit: d9a0b2d
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:13:56 UTC; biocbuild
