import(methods)

importFrom("grDevices", "colorRampPalette", "rainbow", "x11")
importFrom("graphics", "abline", "box", "image", "legend", "lines", "mtext", "par", "plot", "points")
importFrom("stats", "approx", "density", "lowess", "median", "sd")
importFrom("utils", "data", "read.table", "write.table")

importFrom(BiocGenerics, "annotation", "annotation<-", "normalize")
importFrom(annotate, "lookUp", "htmlpage")
importFrom(Biobase, 
		   "phenoData", "phenoData<-",
		   "featureData", "featureData<-",
		   "featureNames", "featureNames<-",
		   "experimentData", "experimentData<-",
		   "sampleNames", "sampleNames<-",
		   "assayDataElement", "assayDataElement<-",
		   "pData", "varMetadata", "exprs")
importFrom(limma,
		   "backgroundCorrect.matrix",
		   "normalizeCyclicLoess",
		   "normalizeMedianValues",
		   "normalizeQuantiles")

importClassesFrom(Biobase, "eSet", "ExpressionSet", "VersionedBiobase", "Versioned")

exportClasses(CodelinkSet, CodelinkSetUnique)
exportMethods(codCorrect, codNormalize, codPreprocess, probeNames, probeTypes,
	getSNR,	getInt, getBkg, getInfo, getWeight, getFlag, meanSNR, codPlot, exprs, chipDevice,
	filterSNR, codUniverse, writeCluster, writeCodelink, averageProbes)

S3method(as.matrix, Codelink)
S3method(dim, Codelink)

exportClasses(Codelink)
exportMethods("[", show, summary, summaryFlag, annotation)

export(readCodelink, writeCodelink, reportCodelink,
	plotMA, plotCorrelation, plotDensities, plotCV, imageCodelink,
	bkgdCorrect, normalize, mergeArray, createWeights, arrayNew,
	selCV, cutCV, fc2Cond, na2false, logCodelink,
	readCodelinkSet,Codelink2CodelinkSet)
